function loadMore(type)
{
    var method = 'browseArchived' + type;
    var selector = '#more' + type + 's';
    var link = createLink('kanban', method, 'projectID=' + v.projectID);
    $(selector).load(link, function()
    {
        var windowHeight = $(window).height();
        var navbarHeight = $('#mainNavbar').height();
        $(selector + ' .panel-body').css('height', windowHeight - 100);
        $(selector + ' .avatar').renderAvatar();
        $(selector).animate({right: 0}, 500);
    });
}

/**
 * Display the kanban in full screen.
 *
 * @access public
 * @return void
 */
function fullScreen()
{
    var element       = document.getElementById('mainContent');
    var requestMethod = element.requestFullScreen || element.webkitRequestFullScreen || element.mozRequestFullScreen || element.msRequestFullscreen;
    if(requestMethod)
    {
        var afterEnterFullscreen = function()
        {
            $('#mainContent').addClass('scrollbar-hover');
            $.cookie('isFullScreen', 1);
        };

        var whenFailEnterFullscreen = function(error)
        {
            $.cookie('isFullScreen', 0);
        };

        try
        {
            var result = requestMethod.call(element);
            if(result && (typeof result.then === 'function' || result instanceof window.Promise))
            {
                result.then(afterEnterFullscreen).catch(whenFailEnterFullscreen);
            }
            else
            {
                afterEnterFullscreen();
            }
        }
        catch (error)
        {
            whenFailEnterFullscreen(error);
        }
    }
}

/**
 * Exit full screen.
 *
 * @access public
 * @return void
 */
function exitFullScreen()
{
    $('#mainContent').removeClass('scrollbar-hover');
    $.cookie('isFullScreen', 0);
}

document.addEventListener('fullscreenchange', function (e)
{
    if(!document.fullscreenElement) exitFullScreen();
});

document.addEventListener('webkitfullscreenchange', function (e)
{
    if(!document.webkitFullscreenElement) exitFullScreen();
});

document.addEventListener('mozfullscreenchange', function (e)
{
    if(!document.mozFullScreenElement) exitFullScreen();
});

document.addEventListener('msfullscreenChange', function (e)
{
    if(!document.msfullscreenElement) exitFullScreen();
});

/**
 * Render WIP of Kanban column.
 */
function renderHeaderCol($column, column)
{
    if(!$column.children('.actions').length) $column.append('<div class="actions"></div>')

    var $actions = $column.children('.actions');
    $actions.attr('title', v.lang.more);
    $actions.html('<button class="btn btn-link action" data-contextmenu="column" data-column="' + column.id + '" data-kanban="' + kanban.id + '"><i class="icon icon-more-v"></i></button>');

    var limit = !column.limit || column.limit == '0' ? '∞' :column.limit;
    if($column.find('.limit').length)
    {
        $column.find('.limit').html(limit);
    }
    else
    {
        $column.find('.count').after("<span class='divide'> / </span><span class='limit'>" + limit + "</span>");
    }
}

/**
 * Render Kanban count
 */
function renderCount($count, count, column)
{
    if(column.limit != 0 && column.limit < count)
    {
        $count.parents('.title').css('background-color', '#F6A1A1');
        $count.css('color', '#E33030');
        if(!$count.parent().find('.error').length) $count.parent().find('.limit').after("<span class='error'>(" + v.kanbanLang.limitExceeded + ")</span>");
    }
    else
    {
        $count.parents('.title').css('background-color', 'transparent');
        $count.css('color', '#8b91a2');
        $count.parent().find('.error').remove();
    }
}

async function renderLaneName($lane, lane, $kanban, columns, kanban)
{
    var privs = await checkPrivs('lane', lane.id, 'editlane,deletelane');
    if(!$lane.children('.actions').length && privs.length)
    {
        $([
          '<div class="actions" title="' + v.lang.more + '">',
          '<a data-contextmenu="lane" data-lane="' + lane.id + '" data-kanban="' + kanban.id + '">',
          '<i class="icon icon-more-v"></i>',
          '</a>',
          '</div>'
        ].join('')).appendTo($lane);
    }
}

/**
 * The function for rendering kanban item
 */
function renderKanbanItem(item, $item)
{
    if(item.status == 'done' && v.taskStatus != 'classic')
    {
        $item.addClass('gray');
    }
    var $title = $item.children('.title');
    if(!$title.length)
    {
        $title = $('<a class="title" data-toggle="modal" data-width="80%"></a>')
            .appendTo($item);
    }
    $title.text(item.name);
    $title.attr('href', createLink('sys.task', 'view', 'taskID=' + item.id + '&from=kanban'));

    var $more = $item.children('.actions');
    if(!$more.length)
    {
        $(
        [
            '<div class="actions" title="' + v.lang.more + '">',
              '<a data-contextmenu="task" data-id="' + item.id + '" data-status="' + item.status + '">',
                '<i class="icon icon-more-v"></i>',
              '</a>',
            '</div>'
        ].join('')).appendTo($item);
    }

    var $info = $item.children('.info');
    if(!$info.length) $info = $(
    [
        '<div class="info">',
            '<span class="pri"></span>',
            '<span class="time label label-light"></span>',
            '<div class="user"></div>',
        '</div>'
    ].join('')).appendTo($item);

    $item.data('task', item);

    $info.children('.pri')
        .attr('class', 'pri label-pri label-pri-' + item.pri)
        .text(item.pri);

    var $time = $info.children('.time');
    if(item.deadline && item.deadline !== '0000-00-00')
    {
        var deadline = $.zui.createDate(item.deadline);
        var today    = $.zui.createDate(v.today);
        var isExpired = deadline.getTime() < today.getTime();
        var dateFormat = (today.getFullYear() === deadline.getFullYear() ? 'MM-dd ' : 'yyyy-MM-dd ') + v.kanbanLang.deadline;
        $time.text($.zui.formatDate(deadline, dateFormat))
            .toggleClass('text-red', isExpired)
            .show();
    }
    else
    {
        $time.hide();
    }

    var $user = $info.children('.user');
    if(item.assignedTo)
    {
        var member = v.members[item.assignedTo] || {account: item.assignedTo};
        var canAssign = item.status !== 'closed' && item.status !== 'cancel';
        var $avatar = $(
            canAssign ? '<a class="avatar avatar-sm avatar-user" data-toggle="modal"></a>'
                : '<div class="avatar avatar-sm avatar-user"></div>'
            ).attr('data-name', member.realname || member.account)
            .attr('data-id', member.id || member.account);
        if(member.avatar)
        {
            $avatar.append('<img src="' + member.avatar + '" />');
        }
        $avatar.renderAvatar();
        if(canAssign)
        {
            $avatar.attr('href', createLink('sys.task', 'assignto', 'taskID=' + item.id + '&fromKanban=1'));
        }

        $user.empty()
            .append($avatar)
            .attr('title', member.realname || member.account)
            .show();
    }
    else
    {
        $user.hide();
    }
}

/**
 * Show error message
 * @param {string|object} message Message
 */
function showErrorMessager(message)
{
    var html = false;
    if(message instanceof Error)
    {
        message = message.message;
    }
    else if(typeof message === 'object')
    {
        html = [];
        $.each(message, function(key, msg)
        {
            html.push($.isArray(msg) ? msg.join('') : String(msg));
        });
        message = html.join('<br/>');
    }
    else
    {
        message = String(message);
    }

    if(typeof message === 'string' && message.length)
    {
        $.zui.messager.danger(message, {html: !!html});
    }
}

/**
 * Update kanban
 * @param {string} [regionID] Region ID
 * @param {string} [groupID] Group ID
 * @param {function(Error)} [callback] Callback on completed
 */
function updateRegion(regionID, groupID, callback)
{
    if(!regionID) return false;
    if(typeof groupID == 'function')
    {
        callback = groupID;
        groupID = 0;
    }

    var url = createLink('kanban', 'ajaxGetData', 'projectID=' + v.projectID + '&regionID=' + regionID + '&group=' + (groupID || 0));
    $.ajax(
    {
        method:   'get',
        dataType: 'json',
        url:      url,
        success: function(data)
        {
            var kanban = $('#kanban' + regionID).data('zui.kanban');
            if(groupID) kanban.renderKanban(data);
            else kanban.render(data);
            typeof callback === 'function' && callback();
        },
        error: function(xhr, status, error)
        {
            showErrorMessager(error || v.lang.timeout);
            typeof callback === 'function' && callback(error);
        }
    });
}

/**
 * Hide kanban action
 */
function hideKanbanAction()
{
    $('.kanban').attr('data-action-enabled', null);
    $('.contextmenu').removeClass('contextmenu-show');
    $('.contextmenu').removeClass('contextmenu-show');
    $('.contextmenu .contextmenu-menu').removeClass('open').removeClass('in');
    $('#moreTasks').animate({right: -400}, 500);
    $('#moreColumns').animate({right: -400}, 500);
}

/**
 * Check action priv
 */
async function checkPrivs(objectType, objectID, actions)
{
    await $.getJSON(createLink('kanban', 'ajaxCheckPriv', `objectType=${objectType}&objectID=${objectID}&action=${actions}`), function(response)
    {
        actions = response.actions;
    });

    return actions;
}

/**
 * Add task to kanban
 * @param {Object} task The new task object
 * @param {string} regionID Region ID
 * @param {string} kanbanID Kanban ID
 * @param {function(Error)} callback Callback on complted
 */
function addTaskToKanban(task, regionID, kanbanID, callback)
{
    var url = createLink('task', 'create', 'projectID=' + v.projectID);
    $.ajax(
    {
        method:   'post',
        dataType: 'json',
        url:      url,
        data:     task,
        success: function(data)
        {
            if(data && data.result === 'success')
            {
                updateRegion(regionID, kanbanID, callback);
            }
            else
            {
                callback && callback(data.message || data);
            }
        },
        error: function(xhr, status, error)
        {
            callback && callback(error || v.lang.timeout);
        }
    });
}

/**
 * Open form for adding task
 * @param {JQuery} $element Trigger element
 */
function openAddTaskForm($element)
{
    var $col  = $element.closest('.kanban-col');
    var $form = $('#kanbanAddTaskForm').appendTo($col);

    if(!$form.data('inited'))
    {
        $form.data('inited', true);
        $form.on('click', '.action-submit', function()
        {
            var $name = $form.find('input[name="name"]');
            var name = $name.val();
            if(!name.length)
            {
                $name.focus().parent().addClass('has-error');
                return;
            }

            $form.addClass('loading');

            var regionID   = $form.closest('.kanban').data('id');
            var kanbanID   = $form.closest('.kanban-board').data('id');
            var columnID   = $form.closest('.kanban-col').data('id');
            var laneID     = $form.closest('.kanban-lane').data('id');
            var statusType = $form.closest('.kanban-col').data('type');

            var task =
            {
                name:         name,
                desc:         $form.find('[name="desc"]').val(),
                pri:          $form.find('[name="pri"]').val(),
                assignedTo:   $form.find('[name="assignedTo"]').val(),
                estimate:     $form.find('[name="estimate"]').val(),
                deadline:     $form.find('[name="deadline"]').val(),
                status:       v.taskStatus == 'classic' ? statusType : 'wait',
                kanbanRegion: regionID,
                kanbanLane:   laneID,
                kanbanColumn: columnID
            };
            addTaskToKanban(task, regionID, kanbanID, function(error)
            {
                $form.removeClass('loading');
                if(error)
                {
                    showErrorMessager(error);
                }
                else
                {
                    hideKanbanAction();
                }
            });
        })
    }

    $form.find('[name="desc"]').val('');
    $form.find('[name="pri"]').val(3);
    $form.find('[name="assignedTo"]').val(v.account);
    $form.find('[name="estimate"]').val('');
    $form.find('[name="deadline"]').val('');
    $form.find('[name="name"]').val('').focus();
}

/**
 * Reset lane height according to window height.
 */
function resetLaneHeight()
{
    var maxHeight = '360px';
    if(v.laneCount < 2)
    {
        var windowHeight = $(window).height();
        var marginTop    = $('#mainContent').css('margin-top');
        var headerHeight = $('.kanban > .kanban-board:first > .kanban-header').outerHeight();
        var actionHeight = $('.kanban > .kanban-board:first > .kanban-lane > .kanban-col:first > .kanban-lane-actions').outerHeight();

        maxHeight = windowHeight - parseInt(marginTop) - headerHeight - actionHeight;
    }
    $('.kanban-lane-items').css('max-height', maxHeight);
}

/**
 * Close modal and update kanban data.
 */
function closeModalAndUpdateKanban(regionID)
{
    setTimeout(function(){
        $.zui.closeModal();
        updateRegion(regionID);
    }, 1200);
}

/**
 * Status change map
 */
var statusChangeMap = {
    wait:   ['doing', 'done', 'cancel'],
    doing:  ['done', 'cancel'],
    done:   ['doing', 'closed'],
    cancel: ['doing', 'closed'],
    closed: ['doing']
};

/**
 * Find drop columns
 * @param {JQuery} $element Drag element
 * @param {JQuery} $root Dnd root element
 */
function findDropColumns($element, $root)
{
    var $task  = $element;
    var task   = $task.data('task');
    var status = task.status;
    var $col   = $task.closest('.kanban-col');
    var col    = $col.data();
    var lane   = $col.closest('.kanban-lane').data('lane');
    var allStatusCanChange = statusChangeMap[status];

    return $root.find('.kanban-lane-col:not([data-type="EMPTY"],[data-type=""])').filter(function()
    {
        var $newCol = $(this);
        var newCol = $newCol.data();
        var $newLane = $newCol.closest('.kanban-lane');
        var newLane = $newLane.data('lane');
        var canChangeCol = v.taskStatus == 'classic' ? allStatusCanChange.indexOf(newCol.type) > -1 : true;
        var canChangeLane = newLane.id !== lane.id && col.type === newCol.type;
        if(canChangeCol || canChangeLane)
        {
            $newCol.addClass('can-drop-here');
            return true;
        }
    });
}

/**
 * Handle drop task
 * @param {Object} event Drop event object
 */
function handleDropTask($element, event, kanban)
{
    if(!event.target || !event.isNew) return;

    var $task    = $element;
    var $oldCol  = $task.closest('.kanban-col');
    var $newCol  = $(event.target).closest('.kanban-col');
    var oldCol   = $oldCol.data();
    var newCol   = $newCol.data();
    var oldLane  = $oldCol.closest('.kanban-lane').data('lane');
    var newLane  = $newCol.closest('.kanban-lane').data('lane');
    var kanbanID = $task.closest('.kanban-board').data('id');
    var regionID = $task.closest('.kanban').data('id');

    if(oldCol.id === newCol.id && newLane.id === oldLane.id) return false;

    var newStatus = newCol.type;
    var task = $task.data('task');

    /* Task status not change */
    if(newStatus === task.status && task.kanbanLane !== newLane.id)
    {
        var url = createLink('sys.task', 'edit', 'taskID=' + task.id);
        return $.ajax(
        {
            method:   'post',
            dataType: 'json',
            url:      url,
            data:     {changeLane: true, kanbanLane: newLane.id},
            success: function(data)
            {
                if(data && data.result === 'success')  updateRegion(regionID);
                else showErrorMessager(data && data.message);
            },
            error: function(xhr, status, error)
            {
                showErrorMessager(error || v.lang.timeout);
            }
        });
    }

    /* Show dialog to user for changing status */
    var methodToChangeStatus =
    {
        wait:   'activate',
        doing:  'start',
        done:   'finish',
        cancel: 'cancel',
        closed: 'close',
    };

    if(v.taskStatus == 'classic')
    {
        var modalUrl = createLink('sys.task', methodToChangeStatus[newStatus], 'taskID=' + task.id + '&laneID=' + newLane.id);
        $.zui.modalTrigger.show(
        {
            url: modalUrl,
            onHide: updateRegion(regionID)
        });
    }
    else
    {
        $.getJSON(createLink('sys.task', 'moveInKanban', 'taskID=' + task.id + '&laneID=' + newLane.id + '&columnID=' + newCol.id), function(response){
            if(response.result == 'success') updateRegion(regionID);
        });
    }
}

/**
 * Handle finish drop task
 */
function handleFinishDrop()
{
    $('.kanban').find('.can-drop-here').removeClass('can-drop-here');
}

/**
 * Adjust add button postion in column
 */
function adjustAddBtnPosition($kanban)
{
    if(!$kanban)
    {
        $('.kanban').children('.kanban-board').each(function()
        {
            adjustAddBtnPosition($(this));
        });
        return;
    }

    $kanban.find('.kanban-lane-col:not([data-type="EMPTY"])').each(function()
    {
        var $col = $(this);
        var items = $col.children('.kanban-lane-items')[0];
        $col.toggleClass('has-scrollbar', items.scrollHeight > items.clientHeight);
    });
}

/**
 * Kanban action handlers
 */
var kanbanActionHandlers =
{
    addItem:  openAddTaskForm,
    dropItem: handleDropTask
};

/**
 * Handle kanban action
 */
function handleKanbanAction(action, $element, event, kanban)
{
    $('.kanban').attr('data-action-enabled', action);
    var handler = kanbanActionHandlers[action];
    if(handler) handler($element, event, kanban);
}

function processMinusBtn()
{
    var columnCount = $('#splitTable .child-column').size();
    if(columnCount > 2 && columnCount < 10)
    {
        $('#splitTable .btn-plus').show();
        $('#splitTable .btn-close').show();
    }
    else if(columnCount <= 2)
    {
        $('#splitTable .btn-close').hide();
    }
    else if(columnCount >= 10)
    {
        $('#splitTable .btn-plus').hide();
    }
}

async function createLaneMenu(options)
{
    var $lane   = options.$trigger.closest('.kanban-lane');
    var $kanban = $lane.closest('.kanban');
    var $region = $lane.closest('.region');
    var laneID  = $lane.data('id');
    var privs   = await checkPrivs('lane', laneID, 'editlane,deletelane');

    var items = [];
    if(privs && privs.includes('editlane'))   items.push({label: v.kanbanLang.editLane, icon: '', url: createLink('kanban', 'editLane', 'lane=' + laneID), attrs: {'data-toggle': 'modal'}});
    if(privs && privs.includes('deletelane')) items.push({label: v.kanbanLang.deleteLane, icon: '', url: createLink('kanban', 'deleteLane', 'lane=' + laneID), className: 'confirmer', attrs: {'data-confirmTitle': v.kanbanlaneLang.confirmDelete, 'data-confirmDetail': v.kanbanlaneLang.confirmDeleteDetail}});

    var bounds = options.$trigger[0].getBoundingClientRect();
    items.$options = {x: bounds.right, y: bounds.top};
    return items;
}

async function createTaskMenu(options)
{
    var taskID = options.id;
    var status = options.status;

    var actions = {
        'edit':  {'label': v.taskLang.edit, 'icon': '', 'url': createLink('task', 'edit', "taskID=" + taskID), 'attrs': {'data-toggle': 'modal', 'data-width': '80%'}},
        'start': {'label': v.taskLang.start, 'icon': '', 'url': createLink('task', 'start', "taskID=" + taskID), 'attrs': {'data-toggle': 'modal'}},
        'finish': {'label': v.taskLang.finish, 'icon': '', 'url': createLink('task', 'finish', "taskID=" + taskID), 'attrs': {'data-toggle': 'modal'}},
        'activate': {'label': v.taskLang.activate, 'icon': '', 'url': createLink('task', 'activate', "taskID=" + taskID), 'attrs': {'data-toggle': 'modal'}},
        'cancel': {'label': v.taskLang.cancel, 'icon': '', 'url': createLink('task', 'cancel', "taskID=" + taskID), 'attrs': {'data-toggle': 'modal'}},
        'close': {'label': v.taskLang.close, 'icon': '', 'url': createLink('task', 'close', "taskID=" + taskID), 'attrs': {'data-toggle': 'modal'}},
        'archive': {'label': v.taskLang.archive, 'icon': '', 'url': createLink('task', 'archive', "taskID=" + taskID), 'attrs': {'class': 'confirmer', 'data-confirmTitle': v.taskLang.confirmArchive, 'data-confirmDetail': v.taskLang.confirmArchiveDetail, 'data-confirmButton': v.taskLang.archive, 'data-confirming': v.taskLang.archiving}},
        'delete': {'label': v.taskLang.delete, 'icon': '', 'url': createLink('task', 'delete', "taskID=" + taskID + '&from=kanban'), 'attrs': {'class': 'confirmer', 'data-confirmTitle': v.taskLang.confirmDelete, 'data-confirmDetail': v.taskLang.confirmDeleteDetail}}
    };

    var keys  = Object.keys(actions);
    var privs = await checkPrivs('task', taskID, keys.join(','));

    var items = [];
    if(!privs) return items;

    $.each(actions, function(actionKey, actionItem)
    {
        if(privs.includes(actionKey)) items.push({label: actionItem.label, icon: '', url: actionItem.url, attrs: actionItem.attrs});
    });

    var bounds = options.$trigger[0].getBoundingClientRect();
    items.$options = {x: bounds.right, y: bounds.top};
    return items;
}

async function createColumnMenu(options)
{
    var $column  = options.$trigger.closest('.kanban-col');
    var $kanban  = $column.closest('.kanban');
    var $region  = $column.closest('.region');
    var columnID = $column.data('id');

    var actions = {
        createColumn: "createCol",
        editColumn: "editCol",
        setWIP: "setWIP",
        splitColumn: "splitCol",
        archiveColumn: "archiveCol",
        deleteColumn: "deleteCol",
    };
    var keys  = Object.keys(actions);
    var privs = await checkPrivs('column', columnID, keys.join(','));
    
    var items = [];
    if(!privs) return items;
    if(privs.includes('setWIP')) items.push({label: v.kanbanLang.WIP, icon: '', url: createLink('kanban', 'setWIP', 'columnID=' + columnID), attrs: {'data-toggle': 'modal'}});
    if(privs.includes('editColumn')) items.push({label: v.kanbanLang.editColumn, icon: '', url: createLink('kanban', 'editColumn', 'columnID=' + columnID), attrs: {'data-toggle': 'modal'}});
    if(privs.includes('createColumn'))
    {
        items.push({label: v.kanbanLang.createColumnOnLeft, icon: '', url: createLink('kanban', 'createColumn', 'columnID=' + columnID + '&position=left'), attrs: {'data-toggle': 'modal'}});
        items.push({label: v.kanbanLang.createColumnOnRight, icon: '', url: createLink('kanban', 'createColumn', 'columnID=' + columnID + '&position=right'), attrs: {'data-toggle': 'modal'}});
    }
    if(privs.includes('splitColumn')) items.push({label: v.kanbanLang.splitColumn, icon: '', url: createLink('kanban', 'splitColumn', 'columnID=' + columnID), attrs: {'data-toggle': 'modal'}});
    if(privs.includes('archiveColumn')) items.push({label: v.kanbanLang.archiveColumn, icon: '', url: createLink('kanban', 'archiveColumn', 'columnID=' + columnID), className: 'confirmer',  attrs: {'data-confirmTitle': v.kanbancolumnLang.confirmArchive, 'data-confirmDetail': v.kanbancolumnLang.confirmArchiveDetail, 'data-confirmButton': v.lang.archive, 'data-confirming': v.lang.archiving}});
    if(privs.includes('deleteColumn')) items.push({label: v.kanbanLang.deleteColumn, icon: '', url: createLink('kanban', 'deleteColumn', 'columnID=' + columnID), className: 'confirmer',  attrs: {'data-confirmTitle': v.kanbancolumnLang.confirmDelete, 'data-confirmDetail': v.kanbancolumnLang.confirmDeleteDetail}});

    var bounds = options.$trigger[0].getBoundingClientRect();
    items.$options = {x: bounds.right, y: bounds.top};
    return items;
}

/* Define menu creators */
window.menuCreators =
{
    task: createTaskMenu,
    lane: createLaneMenu,
    column: createColumnMenu
};

/**
 * init Kanban 
 */
function initKanban($kanban)
{
    var id = $kanban.attr('id');

    $kanban.kanban(
    {
        data:              v[id],
        maxColHeight:      510,
        createColumnText:  v.kanbanLang.createColumn,
        addItemText:       v.kanbanLang.addItem,
        itemRender:        renderKanbanItem,
        onAction:          handleKanbanAction,
        onRenderKanban:    adjustAddBtnPosition,
        onRenderLaneName:  renderLaneName,
        onRenderHeaderCol: renderHeaderCol,
        onRenderCount:     renderCount,
        droppable:
        {
            target:       findDropColumns,
            finish:       handleFinishDrop,
            mouseButton: 'left'
        }
    }).on('click', '.action-cancel', hideKanbanAction);
}

/**
 * Init when page ready
 */
$(function()
{
    if($.cookie('isFullScreen') == 1) fullScreen();

    /* Init first kanban */
    $('.kanban').each(function()
    {
        initKanban($(this));
    });

    $('.icon-double-angle-up,.icon-double-angle-down').on('click', function()
    {
        $(this).toggleClass('icon-double-angle-up icon-double-angle-down');
        $(this).parents('.region').find('.kanban').toggle();
        hideKanbanAction();
    });

    $('.region-header').on('click', '.action', hideKanbanAction);
    $('#TRAction').on('click', '.btn', hideKanbanAction);

    if(v.canSortRegion)
    {
        $('#kanban').sortable({
            selector: '.region',
            trigger: '.region-header',
            start: function(e) {
                $('.icon-double-angle-up').attr('class', 'icon-double-angle-down');
                $('.region').find('.kanban').hide();
                hideKanbanAction();
            },
            finish: function(e) {
                e.element.find('.icon-double-angle-down').attr('class', 'icon-double-angle-up');
                e.element.find('.kanban').show();
                var newOrders = [''];
                e.list.each(function(index, data)
                {
                    newOrders.push(data.item.data('id'));
                });
                $.getJSON(createLink('kanban', 'sortRegion', 'orders=' + newOrders.join(',')), function(response){
                    if(response.result == 'fail' && response.message.length)
                    {
                        bootbox.alert(response.message);
                        setTimeout(function(){return location.reload()}, 3000);
                    }
                });
            }
        });
    }

    $('.kanban-board').sortable({
        selector: '.kanban-lane',
        trigger: '.kanban-lane-name',
        finish: function(e){
            var lanes = [];
            e.list.each(function(index, data){
                lanes.push(data.item.data('id'));
            });
            $.getJSON(createLink('kanban', 'sortLane', 'lanes=' + lanes.join(',')), function(response){
                if(response.result == 'fail' && response.message.length)
                {
                    bootbox.alert(response.message);
                    setTimeout(function(){return location.reload()}, 3000);
                }
            });
        }
    });

    /* Hide action box when user click document */
    $(document).on('click', function(e)
    {
        $('.kanban').each(function()
        {
            var currentAction = $(this).kanban().attr('data-action-enabled');
            var canHideAction = (currentAction === 'headerMore' || currentAction === 'editLaneName')
                && !$(e.target).closest('.action,.action-box').length;
            if(canHideAction) hideKanbanAction();
        });
    });

    /* Init contextmenu */
    $('#kanban').on('click', '[data-contextmenu]', function(event)
    {
        var $trigger    = $(this);
        var menuType    = $trigger.data('contextmenu');
        var menuCreator = window.menuCreators[menuType];
        if(!menuCreator) return;

        var options = $.extend({event, $trigger: $trigger}, $trigger.data());
        menuCreator(options).then(function(items)
        {
            if(!items || !items.length) return;

            $.zui.ContextMenu.show(items, items.$options || {event: event});
        });
        hideKanbanAction();
    });

    /* Adjust the add button position on window resize */
    $(window).on('resize', function(a)
    {
        adjustAddBtnPosition();
    });

    resetLaneHeight();

    /* Hide contextmenu when page scroll */
    $(window).on('scroll', function()
    {
        $.zui.ContextMenu.hide();
    });

    $(document).on('click', '#splitTable .btn-plus', function()
    {
        var tr = $(this).closest('tr');
        tr.after($('#childTpl').html().replace(/key/g, v.key));
        tr.next().find('input[name^=color]').colorPicker();
        v.key++;
        processMinusBtn();
        return false;
    });

    /* Remove a trade detail item. */
    $(document).on('click', '#splitTable .btn-close', function()
    {
        $(this).closest('tr').remove();
        processMinusBtn();
        return false;
    });
});
